@@ -2,11 +2,22 @@ from pathlib import Path
 from typing import Optional
 
 import pytest
-from rich.console import RenderableType
+from rich.console import Console, RenderableType
 from rich.text import Span, Text
 
+from wtfis.clients.abuseipdb import AbuseIpDbClient
+from wtfis.clients.greynoise import GreynoiseClient
+from wtfis.clients.ip2whois import Ip2WhoisClient
+from wtfis.clients.ipwhois import IpWhoisClient
+from wtfis.clients.shodan import ShodanClient
+from wtfis.clients.urlhaus import UrlHausClient
+from wtfis.clients.virustotal import VTClient
+from wtfis.handlers.domain import DomainHandler
+from wtfis.handlers.ip import IpAddressHandler
 from wtfis.models.abuseipdb import AbuseIpDb
 from wtfis.models.greynoise import GreynoiseIp
+from wtfis.models.ip2location import Ip2Location
+from wtfis.models.ipinfo import IpInfo
 from wtfis.models.ipwhois import IpWhois
 from wtfis.models.shodan import ShodanIp
 from wtfis.models.urlhaus import UrlHaus
@@ -63,6 +74,16 @@ def greynoise_get(ip, pool) -> GreynoiseIp:
     return GreynoiseIp.model_validate(pool[ip])
 
 
+def ip2location_get(ip, pool) -> Ip2Location:
+    """Mock replacement for Ip2LocationClient().get_ip()"""
+    return Ip2Location.model_validate(pool[ip])
+
+
+def ipinfo_get(ip, pool) -> IpInfo:
+    """Mock replacement for IpInfoClient().get_ip()"""
+    return IpInfo.model_validate(pool[ip])
+
+
 def ipwhois_get(ip, pool) -> IpWhois:
     """Mock replacement for IpWhoisClient().get_ipwhois()"""
     return IpWhois.model_validate(pool[ip])
@@ -112,6 +133,16 @@ def mock_greynoise_get():
     return greynoise_get
 
 
+@pytest.fixture(scope="module")
+def mock_ip2location_get():
+    return ip2location_get
+
+
+@pytest.fixture(scope="module")
+def mock_ipinfo_get():
+    return ipinfo_get
+
+
 @pytest.fixture(scope="module")
 def mock_ipwhois_get():
     return ipwhois_get
@@ -130,3 +161,42 @@ def mock_urlhaus_get():
 @pytest.fixture(scope="module")
 def display_timestamp():
     return timestamp_text
+
+
+def generate_domain_handler(max_resolutions=3):
+    return DomainHandler(
+        entity="www.example[.]com",
+        console=Console(),
+        vt_client=VTClient("dummykey"),
+        ip_geoasn_client=IpWhoisClient(),
+        whois_client=Ip2WhoisClient("dummykey"),
+        shodan_client=ShodanClient("dummykey"),
+        greynoise_client=GreynoiseClient("dummykey"),
+        abuseipdb_client=AbuseIpDbClient("dummykey"),
+        urlhaus_client=UrlHausClient("dummykey"),
+        max_resolutions=max_resolutions,
+    )
+
+
+@pytest.fixture
+def domain_handler():
+    return generate_domain_handler
+
+
+def generate_ip_handler():
+    return IpAddressHandler(
+        entity="1[.]1[.]1[.]1",
+        console=Console(),
+        vt_client=VTClient("dummykey"),
+        ip_geoasn_client=IpWhoisClient(),
+        whois_client=Ip2WhoisClient("dummykey"),
+        shodan_client=ShodanClient("dummykey"),
+        greynoise_client=GreynoiseClient("dummykey"),
+        abuseipdb_client=AbuseIpDbClient("dummykey"),
+        urlhaus_client=UrlHausClient("dummykey"),
+    )
+
+
+@pytest.fixture
+def ip_handler():
+    return generate_ip_handler
