@@ -16,27 +16,35 @@ from rich.progress import (
 )
 
 from wtfis.clients.abuseipdb import AbuseIpDbClient
+from wtfis.clients.base import requests
 from wtfis.clients.greynoise import GreynoiseClient
+from wtfis.clients.ip2location import Ip2LocationClient
 from wtfis.clients.ip2whois import Ip2WhoisClient
+from wtfis.clients.ipinfo import IpInfoClient
 from wtfis.clients.ipwhois import IpWhoisClient
 from wtfis.clients.shodan import ShodanClient
 from wtfis.clients.urlhaus import UrlHausClient
 from wtfis.clients.virustotal import VTClient
 from wtfis.config import Config, parse_args, parse_env
-from wtfis.exceptions import WtfisException
+from wtfis.exceptions import HandlerException, WtfisException
 from wtfis.handlers.domain import DomainHandler
 from wtfis.handlers.ip import IpAddressHandler
-from wtfis.main import generate_entity_handler, generate_view, main
+from wtfis.main import fetch_data, generate_entity_handler, generate_view, main
 from wtfis.models.virustotal import Domain, IpAddress
 from wtfis.ui.view import DomainView, IpAddressView
 
+# pylint: disable=protected-access,redefined-outer-name
+
 POSSIBLE_ENV_VARS = [
     "VT_API_KEY",
+    "IP2LOCATION_API_KEY",
     "IP2WHOIS_API_KEY",
     "SHODAN_API_KEY",
     "GREYNOISE_API_KEY",
     "ABUSEIPDB_API_KEY",
+    "URLHAUS_API_KEY",
     "WTFIS_DEFAULTS",
+    "GEOLOCATION_SERVICE",
 ]
 
 
@@ -74,10 +82,12 @@ def simulate_progress(console):
 def fake_load_dotenv_1(tmp_path):
     fake_env_vars = {
         "VT_API_KEY": "foo",
+        "IP2LOCATION_API_KEY": "foobar",
         "IP2WHOIS_API_KEY": "alice",
         "SHODAN_API_KEY": "hunter2",
         "GREYNOISE_API_KEY": "upupdowndown",
         "ABUSEIPDB_API_KEY": "dummy",
+        "URLHAUS_API_KEY": "eve",
     }
     return fake_load_dotenv(tmp_path, fake_env_vars)
 
@@ -137,6 +147,7 @@ def fake_load_dotenv_all_ok(tmp_path):
         "GREYNOISE_API_KEY": "bar",
         "ABUSEIPDB_API_KEY": "dummy",
         "IP2WHOIS_API_KEY": "alice",
+        "URLHAUS_API_KEY": "eve",
         "WTFIS_DEFAULTS": "-A",
     }
     return fake_load_dotenv(tmp_path, fake_env_vars)
@@ -152,6 +163,15 @@ def fake_load_dotenv_all_invalid(tmp_path):
     return fake_load_dotenv(tmp_path, fake_env_vars)
 
 
+@pytest.fixture()
+def fake_load_dotenv_geolocation_service_invalid(tmp_path):
+    fake_env_vars = {
+        "VT_API_KEY": "foo",
+        "GEOLOCATION_SERVICE": "invalid_service",
+    }
+    return fake_load_dotenv(tmp_path, fake_env_vars)
+
+
 class TestArgs:
     def test_basic(self):
         with patch(
@@ -218,7 +238,7 @@ class TestArgs:
             capture.err
             == "usage: main [-h]\nmain: error: Maximum --max-resolutions value is 10\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
 
     def test_max_resolutions_error_2(self, capsys):
@@ -240,7 +260,7 @@ class TestArgs:
             "usage: main [-h]\nmain: error: --max-resolutions is not "
             "applicable to IPs\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
 
     def test_shodan_ok(self):
@@ -274,7 +294,7 @@ class TestArgs:
         assert (
             capture.err == "usage: main [-h]\nmain: error: SHODAN_API_KEY is not set\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
 
     def test_greynoise_ok(self):
@@ -309,10 +329,11 @@ class TestArgs:
             capture.err
             == "usage: main [-h]\nmain: error: GREYNOISE_API_KEY is not set\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
 
-    def test_urlhaus(self):
+    def test_urlhaus_ok(self):
+        os.environ["URLHAUS_API_KEY"] = "foo"
         with patch(
             "sys.argv",
             [
@@ -323,6 +344,27 @@ class TestArgs:
         ):
             args = parse_args()
             assert args.use_urlhaus is True
+        del os.environ["URLHAUS_API_KEY"]
+
+    def test_urlhaus_error(self, capsys):
+        with pytest.raises(SystemExit) as e:
+            with patch(
+                "sys.argv",
+                [
+                    "main",
+                    "www.example.com",
+                    "-u",
+                ],
+            ):
+                parse_args()
+
+        capture = capsys.readouterr()
+
+        assert (
+            capture.err == "usage: main [-h]\nmain: error: URLHAUS_API_KEY is not set\n"
+        )
+        assert e.type is SystemExit
+        assert e.value.code == 2
 
     def test_abuseipdb_ok(self):
         os.environ["ABUSEIPDB_API_KEY"] = "foo"
@@ -356,7 +398,7 @@ class TestArgs:
             capture.err
             == "usage: main [-h]\nmain: error: ABUSEIPDB_API_KEY is not set\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
 
     def test_all_ok(self):
@@ -394,16 +436,48 @@ class TestArgs:
             "usage: main [-h]\n"
             "main: error: --use-* flags are not accepted when the --all/-A flag is set\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
         del os.environ["SHODAN_API_KEY"]
 
+    def test_ip2location_ok(self):
+        os.environ["IP2LOCATION_API_KEY"] = "foo"
+        with patch(
+            "sys.argv",
+            [
+                "main",
+                "1.1.1.1",
+                "--geolocation-service",
+                "ip2location",
+            ],
+        ):
+            args = parse_args()
+            assert args.geolocation_service == "ip2location"
+        del os.environ["IP2LOCATION_API_KEY"]
+
+    def test_ip2location_error(self, capsys):
+        with pytest.raises(SystemExit) as e:
+            with patch(
+                "sys.argv",
+                ["main", "1.1.1.1", "--geolocation-service", "ip2location"],
+            ):
+                parse_args()
+
+        capture = capsys.readouterr()
+
+        assert capture.err == (
+            "usage: main [-h]\n" "main: error: IP2LOCATION_API_KEY is not set\n"
+        )
+        assert e.type is SystemExit
+        assert e.value.code == 2
+
 
 class TestEnvs:
     def test_env_file(self, fake_load_dotenv_1):
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
             parse_env()
             assert os.environ["VT_API_KEY"] == "foo"
+            assert os.environ["IP2LOCATION_API_KEY"] == "foobar"
             assert os.environ["IP2WHOIS_API_KEY"] == "alice"
             assert os.environ["SHODAN_API_KEY"] == "hunter2"
             assert os.environ["GREYNOISE_API_KEY"] == "upupdowndown"
@@ -431,9 +505,31 @@ class TestEnvs:
             f"Env file {Path().home() / '.env.wtfis'} was not found either. "
             "Did you forget?\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 1
 
+    @patch("sys.argv", ["main", "1.1.1.1"])
+    def test_geolocation_service_invalid(
+        self, capsys, fake_load_dotenv_geolocation_service_invalid
+    ):
+        with patch(
+            "wtfis.config.load_dotenv", fake_load_dotenv_geolocation_service_invalid
+        ):
+            with pytest.raises(SystemExit) as e:
+                parse_env()
+                parse_args()
+
+        capture = capsys.readouterr()
+
+        assert capture.err == (
+            "usage: main [-h]\n"
+            "main: error: Invalid geolocation service: invalid_service. "
+            "Valid services are: ip2location, ipinfo, ipwhois\n"
+        )
+        assert e.type is SystemExit
+        assert e.value.code == 2
+        unset_env_vars()
+
 
 class TestDefaults:
     def test_defaults_1(self, fake_load_dotenv_2):
@@ -450,10 +546,11 @@ class TestDefaults:
                 assert conf.max_resolutions == 3
                 assert conf.no_color is False
                 assert conf.one_column is True
-                assert conf.use_shodan is True
-                assert conf.use_greynoise is False
-                assert conf.use_abuseipdb is False
-                assert conf.use_urlhaus is False
+                assert isinstance(conf.shodan_client, ShodanClient)
+                assert conf.greynoise_client is None
+                assert conf.abuseipdb_client is None
+                assert conf.urlhaus_client is None
+                assert isinstance(conf.ip_geoasn_client, IpWhoisClient)
                 assert conf.vt_api_key == "foo"
                 assert conf.shodan_api_key == "hunter2"
                 assert conf.abuseipdb_api_key == ""
@@ -476,9 +573,10 @@ class TestDefaults:
                 assert conf.max_resolutions == 3
                 assert conf.no_color is False
                 assert conf.one_column is True
-                assert conf.use_shodan is False
-                assert conf.use_greynoise is False
-                assert conf.use_urlhaus is False
+                assert conf.shodan_client is None
+                assert conf.greynoise_client is None
+                assert conf.urlhaus_client is None
+                assert isinstance(conf.ip_geoasn_client, IpWhoisClient)
         unset_env_vars()
 
     def test_defaults_3(self, fake_load_dotenv_3):
@@ -495,9 +593,9 @@ class TestDefaults:
                 assert conf.max_resolutions == 3
                 assert conf.no_color is True
                 assert conf.one_column is False
-                assert conf.use_shodan is False
-                assert conf.use_greynoise is False
-                assert conf.use_urlhaus is False
+                assert conf.shodan_client is None
+                assert conf.greynoise_client is None
+                assert conf.urlhaus_client is None
         unset_env_vars()
 
     def test_defaults_4(self, fake_load_dotenv_4):
@@ -514,10 +612,10 @@ class TestDefaults:
                 assert conf.entity == "1.1.1.1"
                 assert conf.no_color is False
                 assert conf.one_column is False
-                assert conf.use_shodan is False
-                assert conf.use_greynoise is True
-                assert conf.use_urlhaus is False
-                assert conf.use_abuseipdb is True
+                assert conf.shodan_client is None
+                assert conf.urlhaus_client is None
+                assert isinstance(conf.greynoise_client, GreynoiseClient)
+                assert isinstance(conf.abuseipdb_client, AbuseIpDbClient)
                 assert conf.abuseipdb_api_key == "dummy"
         unset_env_vars()
 
@@ -535,11 +633,10 @@ class TestAllFlag:
                 ],
             ):
                 conf = Config()
-                assert conf.use_shodan is True
-                assert conf.use_greynoise is True
-                assert conf.use_urlhaus is True
-                assert conf.use_abuseipdb is True
-                assert conf.use_urlhaus is True
+                assert isinstance(conf.shodan_client, ShodanClient)
+                assert isinstance(conf.greynoise_client, GreynoiseClient)
+                assert isinstance(conf.abuseipdb_client, AbuseIpDbClient)
+                assert isinstance(conf.urlhaus_client, UrlHausClient)
         unset_env_vars()
 
     def test_2_ok(self, fake_load_dotenv_all_ok):
@@ -553,11 +650,10 @@ class TestAllFlag:
                 ],
             ):
                 conf = Config()
-                assert conf.use_shodan is True
-                assert conf.use_greynoise is True
-                assert conf.use_urlhaus is True
-                assert conf.use_abuseipdb is True
-                assert conf.use_urlhaus is True
+                assert isinstance(conf.shodan_client, ShodanClient)
+                assert isinstance(conf.greynoise_client, GreynoiseClient)
+                assert isinstance(conf.abuseipdb_client, AbuseIpDbClient)
+                assert isinstance(conf.urlhaus_client, UrlHausClient)
         unset_env_vars()
 
     def test_3_error(self, fake_load_dotenv_1, capsys):
@@ -581,7 +677,7 @@ class TestAllFlag:
             "usage: main [-h]\n"
             "main: error: --use-* flags are not accepted when the --all/-A flag is set\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
         unset_env_vars()
 
@@ -604,7 +700,7 @@ class TestAllFlag:
             "usage: main [-h]\n"
             "main: error: --use-* flags are not accepted when the --all/-A flag is set\n"
         )
-        assert e.type == SystemExit
+        assert e.type is SystemExit
         assert e.value.code == 2
         unset_env_vars()
 
@@ -618,13 +714,11 @@ class TestGenEntityHandler:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
             conf = Config()
             console = Console()
-            progress = (simulate_progress(console),)
-            entity = generate_entity_handler(conf, console, progress)
+            entity = generate_entity_handler(conf, console)
         assert isinstance(entity, DomainHandler)
         assert entity.entity == "www.example.com"
         assert entity.max_resolutions == 3
         assert entity.console == console
-        assert entity.progress == progress
         assert isinstance(entity._vt, VTClient)
         assert isinstance(entity._geoasn, IpWhoisClient)
         assert isinstance(entity._whois, Ip2WhoisClient)
@@ -640,8 +734,7 @@ class TestGenEntityHandler:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
             conf = Config()
             console = Console()
-            progress = (simulate_progress(console),)
-            entity = generate_entity_handler(conf, console, progress)
+            entity = generate_entity_handler(conf, console)
         assert entity.max_resolutions == 5
         assert isinstance(entity._geoasn, IpWhoisClient)
         assert isinstance(entity._whois, Ip2WhoisClient)
@@ -656,8 +749,7 @@ class TestGenEntityHandler:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_vt_whois):
             conf = Config()
             console = Console()
-            progress = (simulate_progress(console),)
-            entity = generate_entity_handler(conf, console, progress)
+            entity = generate_entity_handler(conf, console)
         assert isinstance(entity._whois, VTClient)
         unset_env_vars()
 
@@ -667,8 +759,7 @@ class TestGenEntityHandler:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_ip2whois):
             conf = Config()
             console = Console()
-            progress = (simulate_progress(console),)
-            entity = generate_entity_handler(conf, console, progress)
+            entity = generate_entity_handler(conf, console)
         assert isinstance(entity._whois, Ip2WhoisClient)
         unset_env_vars()
 
@@ -678,12 +769,10 @@ class TestGenEntityHandler:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
             conf = Config()
             console = Console()
-            progress = (simulate_progress(console),)
-            entity = generate_entity_handler(conf, console, progress)
+            entity = generate_entity_handler(conf, console)
         assert isinstance(entity, IpAddressHandler)
         assert entity.entity == "1.1.1.1"
         assert entity.console == console
-        assert entity.progress == progress
         assert isinstance(entity._vt, VTClient)
         assert isinstance(entity._geoasn, IpWhoisClient)
         assert isinstance(entity._whois, VTClient)
@@ -697,8 +786,7 @@ class TestGenEntityHandler:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
             conf = Config()
             console = Console()
-            progress = (simulate_progress(console),)
-            entity = generate_entity_handler(conf, console, progress)
+            entity = generate_entity_handler(conf, console)
         assert isinstance(entity._geoasn, IpWhoisClient)
         assert isinstance(entity._whois, VTClient)
         assert isinstance(entity._shodan, ShodanClient)
@@ -707,6 +795,26 @@ class TestGenEntityHandler:
         assert isinstance(entity._abuseipdb, AbuseIpDbClient)
         unset_env_vars()
 
+    @patch("sys.argv", ["main", "1.1.1.1", "--geolocation-service", "ip2location"])
+    def test_handler_ip_3(self, fake_load_dotenv_1):
+        """IP with ip2location geolocation service"""
+        with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
+            conf = Config()
+            console = Console()
+            entity = generate_entity_handler(conf, console)
+        assert isinstance(entity._geoasn, Ip2LocationClient)
+        unset_env_vars()
+
+    @patch("sys.argv", ["main", "1.1.1.1", "--geolocation-service", "ipinfo"])
+    def test_handler_ip_4(self, fake_load_dotenv_1):
+        """IP with ipinfo geolocation service"""
+        with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
+            conf = Config()
+            console = Console()
+            entity = generate_entity_handler(conf, console)
+        assert isinstance(entity._geoasn, IpInfoClient)
+        unset_env_vars()
+
 
 class TestGenView:
     """Tests for the generate_view function"""
@@ -717,7 +825,6 @@ class TestGenView:
         entity = DomainHandler(
             entity=MagicMock(),
             console=MagicMock(),
-            progress=MagicMock(),
             vt_client=MagicMock(),
             ip_geoasn_client=MagicMock(),
             whois_client=MagicMock(),
@@ -740,7 +847,6 @@ class TestGenView:
         entity = IpAddressHandler(
             entity=MagicMock(),
             console=MagicMock(),
-            progress=MagicMock(),
             vt_client=MagicMock(),
             ip_geoasn_client=MagicMock(),
             whois_client=MagicMock(),
@@ -778,9 +884,76 @@ class TestMain:
         with patch("wtfis.config.load_dotenv", fake_load_dotenv_1):
             m_conf.return_value = Config()
             main()
-        m_handler.assert_called_once_with(m_conf(), m_console(), m_progress())
+        m_handler.assert_called_once_with(m_conf(), m_console())
         m_handler().fetch_data.assert_called_once_with()
         m_handler().print_warnings.assert_called_once_with()
         m_view.assert_called_once_with(m_conf(), m_console(), m_handler())
         m_view().print.assert_called_once_with(one_column=False)
         unset_env_vars()
+
+
+class TestFetchData:
+    """
+    Test main.fetch_data()
+    """
+
+    @patch.object(requests.Session, "get")
+    def test_vt_http_error(self, mock_requests_get, domain_handler, capsys):
+        """
+        Test a requests HTTPError from the VT client. This also tests the
+        common_exception_handler decorator.
+        """
+        handler = domain_handler()
+        mock_resp = requests.models.Response()
+
+        mock_resp.status_code = 401
+        mock_requests_get.return_value = mock_resp
+
+        # Thorough test of first _fetch_* method
+        with pytest.raises(SystemExit) as e:
+            fetch_data(MagicMock(), handler)
+
+        capture = capsys.readouterr()
+
+        assert (
+            capture.err == "Error fetching data: 401 Client Error: None for url: None\n"
+        )
+        assert e.type is SystemExit  # ruff E721
+        assert e.value.code == 1
+
+        # Extra: just make sure program exits correctly
+        with pytest.raises(HandlerException) as e:
+            handler._fetch_vt_resolutions()
+
+    @patch.object(requests.Session, "get")
+    def test_vt_validation_error(self, mock_requests_get, domain_handler, capsys):
+        """
+        Test main.fetch_data().
+        Test a pydantic data model ValidationError from the VT client. This also tests
+        the common_exception_handler decorator.
+        """
+        handler = domain_handler()
+        mock_resp = requests.models.Response()
+
+        with patch.object(mock_resp, "json") as mock_resp_json:
+            mock_resp.status_code = 200
+            mock_resp_json.return_value = {"intentionally": "wrong data"}
+            mock_requests_get.return_value = mock_resp
+
+            # Thorough test of first _fetch_* method
+            with pytest.raises(SystemExit) as e:
+                fetch_data(MagicMock(), handler)
+
+            capture = capsys.readouterr()
+
+            assert capture.err.startswith(
+                "Data model validation error: 1 validation error for Domain\ndata\n"
+                "  Field required [type=missing, input_value={'intentionally': "
+                "'wrong data'}, input_type=dict]\n"
+            )
+            assert e.type is SystemExit
+            assert e.value.code == 1
+
+            # Extra: just make sure program exits correctly
+            with pytest.raises(HandlerException) as e:
+                handler._fetch_vt_resolutions()
