@@ -1,5 +1,6 @@
 import abc
-from typing import Callable, List, Optional, Union
+from dataclasses import dataclass
+from typing import Callable, Generator, List, Optional, Tuple, Union
 
 from pydantic import ValidationError
 from requests.exceptions import (
@@ -10,7 +11,6 @@ from requests.exceptions import (
     Timeout,
 )
 from rich.console import Console
-from rich.progress import Progress
 from shodan.exception import APIError
 
 from wtfis.clients.abuseipdb import AbuseIpDbClient
@@ -19,6 +19,7 @@ from wtfis.clients.shodan import ShodanClient
 from wtfis.clients.types import IpGeoAsnClientType, IpWhoisClientType
 from wtfis.clients.urlhaus import UrlHausClient
 from wtfis.clients.virustotal import VTClient
+from wtfis.exceptions import HandlerException
 from wtfis.models.abuseipdb import AbuseIpDbMap
 from wtfis.models.base import WhoisBase
 from wtfis.models.greynoise import GreynoiseIpMap
@@ -28,22 +29,19 @@ from wtfis.models.types import IpGeoAsnMapType
 from wtfis.models.urlhaus import UrlHausMap
 from wtfis.models.virustotal import Domain, IpAddress
 from wtfis.ui.theme import Theme
-from wtfis.utils import error_and_exit, refang
+from wtfis.utils import refang
 
 
 def common_exception_handler(func: Callable) -> Callable:
     """Decorator for handling common fetch errors"""
 
     def inner(*args, **kwargs) -> None:
-        progress: Progress = args[0].progress  # args[0] is the method's self input
         try:
             func(*args, **kwargs)
         except (APIError, ConnectionError, HTTPError, JSONDecodeError, Timeout) as e:
-            progress.stop()
-            error_and_exit(f"Error fetching data: {e}")
+            raise HandlerException(f"Error fetching data: {e}") from e
         except ValidationError as e:
-            progress.stop()
-            error_and_exit(f"Data model validation error: {e}")
+            raise HandlerException(f"Data model validation error: {e}") from e
 
     return inner
 
@@ -66,33 +64,30 @@ def failopen_exception_handler(client_attr_name: str) -> Callable:
     return inner
 
 
+@dataclass
 class BaseHandler(abc.ABC):
-    def __init__(
-        self,
-        entity: str,
-        console: Console,
-        progress: Progress,
-        vt_client: VTClient,
-        ip_geoasn_client: IpGeoAsnClientType,
-        whois_client: IpWhoisClientType,
-        shodan_client: Optional[ShodanClient],
-        greynoise_client: Optional[GreynoiseClient],
-        abuseipdb_client: Optional[AbuseIpDbClient],
-        urlhaus_client: Optional[UrlHausClient],
-    ):
+    entity: str
+    console: Console
+    vt_client: VTClient
+    ip_geoasn_client: IpGeoAsnClientType
+    whois_client: IpWhoisClientType
+    shodan_client: Optional[ShodanClient]
+    greynoise_client: Optional[GreynoiseClient]
+    abuseipdb_client: Optional[AbuseIpDbClient]
+    urlhaus_client: Optional[UrlHausClient]
+
+    def __post_init__(self):
         # Process-specific
-        self.entity = refang(entity)
-        self.console = console
-        self.progress = progress
+        self.entity = refang(self.entity)
 
         # Clients
-        self._vt = vt_client
-        self._geoasn = ip_geoasn_client
-        self._whois = whois_client
-        self._shodan = shodan_client
-        self._greynoise = greynoise_client
-        self._abuseipdb = abuseipdb_client
-        self._urlhaus = urlhaus_client
+        self._vt = self.vt_client
+        self._geoasn = self.ip_geoasn_client
+        self._whois = self.whois_client
+        self._shodan = self.shodan_client
+        self._greynoise = self.greynoise_client
+        self._abuseipdb = self.abuseipdb_client
+        self._urlhaus = self.urlhaus_client
 
         # Dataset containers
         self.vt_info: Union[Domain, IpAddress]
@@ -107,7 +102,7 @@ class BaseHandler(abc.ABC):
         self.warnings: List[str] = []
 
     @abc.abstractmethod
-    def fetch_data(self) -> None:
+    def fetch_data(self) -> Generator[Union[Tuple[str, int], int], None, None]:
         """Main method that controls what get fetched"""
         return NotImplemented  # type: ignore  # pragma: no coverage
 
