@@ -6,6 +6,8 @@ from typing import Union
 
 from wtfis.models.abuseipdb import AbuseIpDbMap
 from wtfis.models.greynoise import GreynoiseIpMap
+from wtfis.models.ip2location import Ip2Location, Ip2LocationMap
+from wtfis.models.ipinfo import IpInfo, IpInfoMap
 from wtfis.models.ipwhois import IpWhois, IpWhoisMap
 from wtfis.models.shodan import ShodanIpMap
 from wtfis.models.urlhaus import UrlHausMap
@@ -14,6 +16,8 @@ from wtfis.models.urlhaus import UrlHausMap
 IpEnrichmentType = Union[
     AbuseIpDbMap,
     GreynoiseIpMap,
+    Ip2LocationMap,
+    IpInfoMap,
     IpWhoisMap,
     ShodanIpMap,
     UrlHausMap,
@@ -23,6 +27,14 @@ IpEnrichmentType = Union[
 DomainEnrichmentType = Union[UrlHausMap,]
 
 # IP geolocation and ASN types
-IpGeoAsnType = Union[IpWhois,]
+IpGeoAsnType = Union[
+    Ip2Location,
+    IpInfo,
+    IpWhois,
+]
 
-IpGeoAsnMapType = Union[IpWhoisMap,]
+IpGeoAsnMapType = Union[
+    Ip2LocationMap,
+    IpInfoMap,
+    IpWhoisMap,
+]
